@@ -1,3 +1,9 @@
+"""Top-level package for apkutils."""
+
+__author__ = """King Orz"""
+__email__ = 'kin9-0rz@outlook.com'
+__version__ = '0.10.2'
+
 import binascii
 import re
 import xml
@@ -6,15 +12,14 @@ from xml.parsers.expat import ExpatError
 import xmltodict
 from anytree import Node, RenderTree
 from anytree.resolver import Resolver
-from apkutils import apkfile
-from apkutils.axml.arscparser import ARSCParser
-from apkutils.axml.axmlparser import AXML
-from apkutils.dex.dexparser import DexFile
 from bs4 import BeautifulSoup
 from cigam import Magic
-from TextWizard import hash
+from lxml import etree
+from petty import hash
 
-__version__ = '0.8.1'
+from apkutils import apkfile
+from apkutils.axml import ARSCParser, AXMLPrinter
+from apkutils.dex.dexparser import DexFile
 
 # 6E invoke-virtual 110
 # 6F invoke-supper
@@ -28,6 +33,9 @@ __version__ = '0.8.1'
 # 78 invoke-interface-range
 INVOKE_OPCODES = {0x6e, 0x6f, 0x70, 0x71, 0x72, 0x74, 0x75, 0x76, 0x77, 0x78}
 
+NS_ANDROID_URI = 'http://schemas.android.com/apk/res/android'
+NS_ANDROID = '{{{}}}'.format(NS_ANDROID_URI)  # Namespace as used by etree
+
 
 class APK:
 
@@ -60,7 +68,7 @@ class APK:
     #     _xml = re.sub(r'"\s+?>', '">', _xml)
     #     _xml = re.sub(r'>\s+?<', '><', _xml)
     #     return _xml
-    
+
     @staticmethod
     def serialize_xml(org_xml):
         _xml = ''
@@ -129,8 +137,10 @@ class APK:
                         name = f['name']
                         if item['@type'] in name and item['@name'] in name:
                             self.app_icon = name
-        except Exception as ex:
-            raise ex
+        except Exception as e:
+            print(self.apk_path)
+            print(e)
+
 
     def get_trees(self, height=2, limit=5000):
         if self.trees is None:
@@ -294,7 +304,7 @@ class APK:
                         # 1B const-string-jumbo
                         if bc.opcode not in {26, 27}:
                             continue
-                        
+
                         if method.id.cname is None:
                             continue
 
@@ -384,8 +394,10 @@ class APK:
                             and Magic(data).get_type() == 'dex':
                         dex_file = DexFile(data)
                         self.dex_files.append(dex_file)
-        except Exception as ex:
-            raise ex
+        except Exception as e:
+            print(self.apk_path)
+            print(e)
+
 
     def get_strings(self):
         if not self.strings:
@@ -439,7 +451,9 @@ class APK:
                     # item["sha1"] = ""
                     self.children.append(item)
         except Exception as e:
-            raise e
+            print(self.apk_path)
+            print(e)
+
 
     def get_org_manifest(self):
         if not self.org_manifest:
@@ -453,18 +467,63 @@ class APK:
                 if ANDROID_MANIFEST in zf.namelist():
                     data = zf.read(ANDROID_MANIFEST)
                     try:
-                        axml = AXML(data)
-                        if axml.is_valid:
-                            self.org_manifest = axml.get_xml()
+                        self.axml = AXMLPrinter(data).get_xml_obj()
+                        buff = etree.tostring(
+                            self.axml, pretty_print=True, encoding="utf-8")
+                        self.org_manifest = buff.decode("UTF-8")
+
                     except Exception as e:
                         raise e
         except Exception as e:
-            raise e
+            print(self.apk_path)
+            print(e)
+
 
         # fix manifest
         self.org_manifest = re.sub(
             r'\s:(="[\w]*?\.[\.\w]*")', r' android:name\1', self.org_manifest)
 
+    def get_main_activities(self):
+        x = set()
+        y = set()
+
+        activities_and_aliases = self.axml.findall(".//activity") + \
+            self.axml.findall(".//activity-alias")
+
+        for item in activities_and_aliases:
+            # Some applications have more than one MAIN activity.
+            # For example: paid and free content
+            activityEnabled = item.get(self._ns("enabled"))
+            if activityEnabled == "false":
+                continue
+
+            for sitem in item.findall(".//action"):
+                val = sitem.get(self._ns("name"))
+                if val == "android.intent.action.MAIN":
+                    activity = item.get(self._ns("name"))
+                    if activity is not None:
+                        x.add(item.get(self._ns("name")))
+                    else:
+                        print('Main activity without name')
+
+            for sitem in item.findall(".//category"):
+                val = sitem.get(self._ns("name"))
+                if val == "android.intent.category.LAUNCHER":
+                    activity = item.get(self._ns("name"))
+                    if activity is not None:
+                        y.add(item.get(self._ns("name")))
+                    else:
+                        print('Launcher activity without name')
+
+        return x.intersection(y)
+
+    @staticmethod
+    def _ns(name):
+        """
+        return the name including the Android namespace URI
+        """
+        return NS_ANDROID + name
+
     def get_manifest(self):
         if not self.manifest:
             self._init_manifest()
@@ -479,7 +538,7 @@ class APK:
                 self.manifest = xmltodict.parse(
                     self.org_manifest, False)['manifest']
             except xml.parsers.expat.ExpatError as e:
-                pass
+                raise e
             except Exception as e:
                 raise e
 
@@ -556,7 +615,9 @@ class APK:
                     data = zf.read(ARSC_NAME)
                     self.arsc = ARSCParser(data)
         except Exception as e:
-            raise e
+            print(self.apk_path)
+            print(e)
+
 
     def get_arsc(self):
         if not self.arsc:
@@ -581,17 +642,17 @@ class APK:
                             cert = Certificate(data, digestalgo=digestalgo)
                             self.certs[digestalgo] = cert.get()
         except Exception as e:
-            raise e
+            print(self.apk_path)
+            print(e)
 
     def get_opcodes(self):
-        if not self.dex_files:
+        if not self.opcodes:
             self._init_opcodes()
         return self.opcodes
 
     def _init_opcodes(self):
         if not self.dex_files:
             self._init_dex_files()
-
         self.opcodes = []
         for dex_file in self.dex_files:
             for dexClass in dex_file.classes:
@@ -599,7 +660,6 @@ class APK:
                     dexClass.parseData()
                 except IndexError:
                     continue
-
                 for method in dexClass.data.methods:
                     opcodes = ""
                     if method.code:
